﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.IO;
using System.Reflection;
using Microsoft.CSharp;
namespace CodeGeneratorTool
{
    public partial class CodeGenerator : Form
    {
        private StringBuilder _Fields;
        private StringBuilder _Properties;
        private StringBuilder _Inits;

        private CodeNamespace _CurrentNamespace;
        private List<AllProperties> properties;
        private AutoCompleteStringCollection _Types;
        private List<string> typeNames;

        public CodeGenerator()
        {
            InitializeComponent();
            this.properties= new List<AllProperties>();
            this._Types = GetPrimitiveTypeList();
            this.typeNames = GetTypeNames();
           
        }
        private void InitContent()
        {
            this._Fields = new StringBuilder();
            this._Properties = new StringBuilder();
            this._Inits = new StringBuilder();
        }

        private void browseClassListExcel_HelpRequest(object sender, EventArgs e)
        {

        }
        /// <summary>
        /// The properties (types and names) of the manual object
        /// </summary>
        public List<AllProperties > Properties
        {
            get { return properties; }
            set { properties = value; }
        }

        private void button1_Click(object sender, EventArgs e)
        {
             createCodeBrowser.ShowDialog();    
             string fileName = createCodeBrowser.FileName;
                      

        }
        private CodeMemberProperty GeneratePublicProperty(Type t, string name)
        {
            CodeMemberProperty cmp = new CodeMemberProperty();
            cmp.Attributes = MemberAttributes.Public | MemberAttributes.Final;
            cmp.Name = name;
            cmp.HasGet = true;
            cmp.HasSet = true;

            CodeTypeReference ctr = new CodeTypeReference(t);
            cmp.Type = ctr;

            string field = GenerateFieldName(name);

            //setup the get
            CodeThisReferenceExpression thisRef = new CodeThisReferenceExpression();
            CodeFieldReferenceExpression fld = new CodeFieldReferenceExpression(thisRef, field);
            CodeMethodReturnStatement get = new CodeMethodReturnStatement(fld);
            cmp.GetStatements.Add(get);

            //setup the set
            CodeArgumentReferenceExpression setVal = new CodeArgumentReferenceExpression("value");
            CodeAssignStatement set = new CodeAssignStatement(fld, setVal);
            cmp.SetStatements.Add(set);

            return cmp;
        }
        private string GenerateFieldName(string Name)
        {
            return ("_" + Name.ToLowerInvariant());
        }

        
        private CodeConstructor GenerateInitMethod()
        {
            CodeConstructor cc = new CodeConstructor();
            cc.Attributes = MemberAttributes.Public | MemberAttributes.Final;

            return cc;
        }
        private CodeNamespace GenerateNamespace()
        {
            //init the namespace
            CodeNamespace ns = new CodeNamespace(textBox2.Text);
            
            
            //add some imports
            ns.Imports.Add(new CodeNamespaceImport("System"));
            ns.Imports.Add(new CodeNamespaceImport("System.Collections.Generic"));
             ns.Imports.Add(new CodeNamespaceImport("System.Text"));
            

            return ns;
        }
           private CodeTypeDeclaration GenerateClassDef()
        {
            //create the class def
            CodeTypeDeclaration typeDec = new CodeTypeDeclaration(textBox3.Text);
            typeDec.Comments.Add(new CodeCommentStatement("Fill Comments for Class " + textBox3.Text, true));
               
            typeDec.IsClass = true;
            typeDec.Attributes = MemberAttributes.Public;
            
               
            return typeDec;
        }
        private CodeMemberField GenerateField(Type t, string name)
        {
            CodeTypeReference ctr = new CodeTypeReference(t);
            CodeMemberField cmf = new CodeMemberField(ctr, name);
            cmf.Attributes = MemberAttributes.Private;

            return cmf;
        }
        
        private CodeAssignStatement GenerateFieldInit(string fieldName, object value, Type fieldType)
        {
            CodeAssignStatement cas;
            CodeThisReferenceExpression ctre = new CodeThisReferenceExpression();
            if (!fieldType.IsArray && fieldType != typeof(Guid))
            {
                CodePrimitiveExpression newVal = new CodePrimitiveExpression(value);
                cas = new CodeAssignStatement(new CodeFieldReferenceExpression(ctre, fieldName), newVal);
            }
            else if (!fieldType.IsArray && fieldType == typeof(Guid))
            {
                CodeObjectCreateExpression newVal = new CodeObjectCreateExpression(fieldType, new CodeExpression[] { new CodePrimitiveExpression(value) });
                cas = new CodeAssignStatement(new CodeFieldReferenceExpression(ctre, fieldName), newVal);
            }
            else
            {
                CodeArrayCreateExpression newVal = new CodeArrayCreateExpression(value.GetType(), 0);
                cas = new CodeAssignStatement(new CodeFieldReferenceExpression(ctre, fieldName), newVal);
            }

            return cas;
        }
         /// <summary>
        /// Get a strongly typed default value for the type passed
        /// </summary>
        /// <param name="t"></param>
        /// <param name="lang"></param>
        /// <returns></returns>
        private object GetTypeDefaultValue(Type t)
        {
            //we really only need to initialize nullable types
            object result = null;
            if (t == typeof(String))
            {
                result = string.Empty;
            }
            else if (t == typeof(Byte[]))
            {
                result = new Byte[] { };
            }
            else if (t == typeof(Object))
            {
                result = new Object();
            }
            else if (t == typeof(Guid))
            {
                result = Guid.Empty.ToString();
            }

            return result;
        }
        /// <summary>
        /// Perform the final code generation
        /// </summary>
        /// <param name="prov"></param>
        /// <returns></returns>
        private string GenerateClassBody(CodeDomProvider prov)
        {
            CodeGeneratorOptions cgo = new CodeGeneratorOptions();
            cgo.BracingStyle = "C"; //"C" braces on next line -- other option "Block" is on the same line
            cgo.IndentString = "\t"; //tab for indents
            cgo.BlankLinesBetweenMembers = false;
            

            //setup the containers for the text of the code
            StringBuilder sb = new StringBuilder();
            StringWriter sw = new StringWriter(sb);

            //perform the generation
            prov.GenerateCodeFromNamespace(this._CurrentNamespace, sw, cgo);

            //return the text of the generated code
            return sb.ToString();
        }
        

        #region Internal Execution Methods
        private void ExecuteBuilder()
        {
            this._CurrentNamespace = GenerateNamespace();
            
            CodeTypeDeclaration cls = GenerateClassDef();
            
            this._CurrentNamespace.Types.Add(cls);

            CodeConstructor init = GenerateInitMethod();
            cls.Members.Add(init);
            CodeCommentStatementCollection comments = new CodeCommentStatementCollection();
            comments.Add(new CodeCommentStatement("This Code Has Been Generated Using CodeGenerator By Soumyajeet Mahapatra Beta Version"
                + "\r\n This Code is Provided as is without any warranties explicit or implied", true));
            

            this._CurrentNamespace.Comments.AddRange(comments);
           
                ExecuteFromForm(cls, init);

            
        }
        private void ExecuteFromForm(CodeTypeDeclaration cls, CodeConstructor init)
        {
            foreach (AllProperties ep in this.Properties)
            {
                Type t = (ep.CLRType == null) ? typeof(object) : ep.CLRType;
                string fieldName = GenerateFieldName(ep.Name);
                cls.Members.Add(GenerateField(t, fieldName));

                object defaultVal = GetTypeDefaultValue(t);

                if (defaultVal != null)
                {
                    //put init code in the constructor for the field
                    init.Statements.Add(GenerateFieldInit(fieldName, defaultVal, t));
                }

                //add a public property
                cls.Members.Add(GeneratePublicProperty(t, ep.Name));
            }
        }
#endregion

        private void button2_Click(object sender, EventArgs e)
        {
            InitContent();
            this.properties = GetProperties();
            ExecuteBuilder();
      
            string classText = GetEntityClassBody();
            richTextBox1.Text = classText;
        }
        #region Public-facing methods
        public string GetEntityClassBody()
        {
            
                return CSharpCode;
        }
        #endregion

        #region Private Properties
        private string CSharpCode
        {
            get
            {
                CSharpCodeProvider provider = new CSharpCodeProvider();
                
                return GenerateClassBody(provider);
            }
        }
        
        #endregion
        public List<AllProperties> GetProperties()
        {
            List<AllProperties> props = new List<AllProperties>();

            foreach (ListViewItem lvi in listView1.Items)
            {
                AllProperties mp = new AllProperties();
                mp.Name = lvi.SubItems[0].Text;
                Type t = Type.GetType(lvi.SubItems[1].Text, false);
                if (t == null)
                    t = Type.GetType(string.Format("System.{0}", lvi.SubItems[1].Text), false);
                mp.CLRType = t;

                props.Add(mp);
            }

            return props;
        }

        private AutoCompleteStringCollection GetPrimitiveTypeList()
        {
            AutoCompleteStringCollection types = new AutoCompleteStringCollection();
            //get the proper names for the types in the System namespace
            foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies())
            {
                if (Path.GetFileName(asm.Location) == "mscorlib.dll")
                {
                    foreach (Type t in asm.GetTypes())
                    {
                        if (!t.IsAbstract)
                            types.Add(t.Name);
                    }
                    break; //don't need to iterate through the rest of the assemblies
                }
            }

            return types;
        }
        private List<string>GetTypeNames()
        {
            List<string> names = new List<string>();
            //get the proper names for the types in the System namespace
            foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies())
            {
                if (Path.GetFileName(asm.Location) == "mscorlib.dll")
                {
                    foreach (Type t in asm.GetTypes())
                    {
                        if (!t.IsAbstract)
                            names.Add(t.Name);
                    }
                    break; //don't need to iterate through the rest of the assemblies
                }
            }

            return names;
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (txtField.Text.Trim() != string.Empty && txtType.Text.Trim() != string.Empty&&this.typeNames.Contains(txtType.Text.Trim() ))
            {
                ListViewItem lvi = new ListViewItem(new string[] { txtField.Text.Trim(), txtType.Text.Trim() });
                listView1.Items.Add(lvi);

                txtField.Text = "";
                txtType.Text = "";

               

                txtType.Focus();
            }
            else
            {
                MessageBox.Show("Please enter a field name and a primitive data type");
            }
        }

        private void btn_Remove_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem lvi in listView1.SelectedItems)
            {
                listView1.Items.Remove(lvi);
            }

        }

        private void btn_Upbtn_Up_Click(object sender, EventArgs e)
        {
            MoveListViewItem(true);
        }

        private void MoveListViewItem(bool moveUp)
        {
            //move the selected items up
            if (listView1.SelectedItems.Count > 0)
            {
                foreach (ListViewItem lvi in listView1.SelectedItems)
                {
                    int oldIndex = lvi.Index;
                    int newIndex;
                    if (moveUp)
                        newIndex = lvi.Index - 1;
                    else
                        newIndex = lvi.Index + 1;

                    //ignore a move up request for the first item and ignore move down request for the last item
                    if (!(oldIndex == 0 && moveUp) && !(oldIndex >= (listView1.Items.Count - 1) && !moveUp))
                    {
                        //cache the item next to the moving item
                        ListViewItem cache = (ListViewItem)listView1.Items[newIndex].Clone();
                        //replace the item next to the moving item with the moving item
                        listView1.Items[newIndex] = (ListViewItem)lvi.Clone();
                        //replace the moving item with the item next to it
                        listView1.Items[oldIndex] = (ListViewItem)cache.Clone();

                        //select the moved item
                        listView1.Items[newIndex].Selected = true;
                        listView1.Refresh();
                    }
                    listView1.Focus();
                }
            }
        }


        private void btn_Down_Click(object sender, EventArgs e)
        {
            MoveListViewItem(false);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Bind_Types();
        }
        private void Bind_Types()
        {
            txtType.AutoCompleteCustomSource = this._Types;
        }
    }

    }
